<?php

namespace Drupal\monarch_hook_plugin;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Extension\ModuleHandler as CoreModuleHandler;

/**
 * The new module handler service.
 */
class ModuleHandler extends CoreModuleHandler {

  /**
   * The hook plugin manager service.
   *
   * @var \Drupal\monarch_hook_plugin\HookPluginManager
   */
  protected $hookPluginManager;

  /**
   * Get the hook plugin manager service.
   *
   * @return \Drupal\monarch_hook_plugin\HookPluginManager
   *   The hook plugin manager service.
   */
  protected function getHookPluginManager() {
    return $this->hookPluginManager = $this->hookPluginManager ?? \Drupal::service('plugin.manager.monarch_hook_plugin.hook');
  }

  /**
   * {@inheritdoc}
   */
  public function invokeAll($hook, array $args = []) {
    $return = [];
    $func_name = $this->getHookPluginManager()->getFunctionName('hook_' . $hook);

    foreach ($this->getHookPluginManager()->getPlugins($func_name, -INF, 0) as $plugin) {
      $result = $plugin->{$func_name}(...$args);
      if (isset($result) && is_array($result)) {
        $return = NestedArray::mergeDeep($return, $result);
      }
      elseif (isset($result)) {
        $return[] = $result;
      }
    }

    $this->invokeAllWith($hook, function (callable $hook, string $module) use ($args, &$return) {
      $result = call_user_func_array($hook, $args);
      if (isset($result) && is_array($result)) {
        $return = NestedArray::mergeDeep($return, $result);
      }
      elseif (isset($result)) {
        $return[] = $result;
      }
    });

    foreach ($this->getHookPluginManager()->getPlugins($func_name, 0, INF) as $plugin) {
      $result = $plugin->{$func_name}(...$args);
      if (isset($result) && is_array($result)) {
        $return = NestedArray::mergeDeep($return, $result);
      }
      elseif (isset($result)) {
        $return[] = $result;
      }
    }

    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function alter($type, &$data, &$context1 = NULL, &$context2 = NULL) {
    $types = is_array($type) ? $type : [$type];

    foreach ($types as $hook) {
      if (is_string($hook)) {
        $func_name = $this->getHookPluginManager()->getFunctionName('hook_' . $hook . '_alter');
        foreach ($this->getHookPluginManager()->getPlugins($func_name, -INF, 0) as $plugin) {
          $plugin->{$func_name}($data, $context1, $context2);
        }
      }
    }

    parent::alter($type, $data, $context1, $context2);

    foreach ($types as $hook) {
      if (is_string($hook)) {
        $func_name = $this->getHookPluginManager()->getFunctionName('hook_' . $hook . '_alter');
        foreach ($this->getHookPluginManager()->getPlugins($func_name, 0, INF) as $plugin) {
          $plugin->{$func_name}($data, $context1, $context2);
        }
      }
    }
  }

}
